/*
  JsUpload: Client Side File Loading
  JavaScript + Flash Library

  Version: 0.2

  Copyright (c) 2009 Douglas C. Neiner

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/
package {
    import flash.system.Security;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.net.FileReference;
    import flash.net.FileFilter;
    import flash.net.URLRequest;
    import flash.display.*;
    import flash.utils.ByteArray;
    import flash.external.ExternalInterface;
    import com.dynamicflash.util.Base64;

    [SWF(backgroundColor="#CCCCCC")]
    [SWF(backgroundAlpha=0)]
    public class JsUpload extends Sprite {
    
    private var options:Object,
                file:FileReference = new FileReference(),
                queue_name:String  = "",
    
                _width:Number      = 0,
                _height:Number     = 0,
                _dataTypeForLoad:String = "",
                                   
                enabled:Boolean    = true,
                over:Boolean       = false,
                down:Boolean       = false,
                
                buttonImage:String = "images/upload.png",
                
                button:Loader;
    
    public function JsUpload() {
        Security.allowDomain('*');

        stage.align = StageAlign.TOP_LEFT;
        stage.scaleMode = StageScaleMode.NO_SCALE;

        options = this.root.loaderInfo.parameters;

        queue_name = options.queue_name.toString();

        _width  = options.width;
        _height = options.height;
        _dataTypeForLoad = options.dataTypeForLoad;

        //ExternalInterface.call('JsUpload.alert', queue_name, options.dataTypeForLoad);

        if (options.uploadImage) {
            buttonImage = options.uploadImage;
        }

        setupDefaultButton();
        addChild(button);

        this.buttonMode = true;

        this.addEventListener(MouseEvent.CLICK, onMouseClickEvent);
        this.addEventListener(MouseEvent.ROLL_OVER , onMouseEnter);
        this.addEventListener(MouseEvent.ROLL_OUT , onMouseLeave);
        this.addEventListener(MouseEvent.MOUSE_DOWN , onMouseDown);
        this.addEventListener(MouseEvent.MOUSE_UP , onMouseUp);

        ExternalInterface.addCallback('setEnabled', setEnabled);

        file.addEventListener(Event.COMPLETE, onLoadComplete);
        file.addEventListener(Event.CANCEL, onLoadCancel);
    }
    
    private function setEnabled(isEnabled:Boolean):Boolean {
        enabled = isEnabled;
        if(enabled === true){
            button.y = 0;
            this.buttonMode = true;
        } else {
            button.y = (-3 * _height);
            this.buttonMode = false;
        }
        return enabled;
    }
    
    private function setupDefaultButton():void {
        button = new Loader();
        var urlReq:URLRequest = new URLRequest(buttonImage);
        button.load(urlReq);
        button.x = 0;
        button.y = 0;
    }
    
    protected function onMouseEnter(event:Event):void {
        if (enabled === true){
            if (down === false) {
                button.y = (-1 * _height);
            }
            over = true;
        }
    }
    
    protected function onMouseLeave(event:Event):void {
        if (enabled === true){
            if (down === false) {
                button.y = 0;
            }
            over = false;
        }
    }
    
    protected function onMouseDown(event:Event):void {
        if(enabled === true){
            button.y = button.y = (-2 * _height);
            down = true;
        }
    }
    
    protected function onMouseUp(event:Event):void {
        if(enabled === true){
            if(over === false){
                button.y = 0;
            } else {
                button.y = (-1 * _height);
            }
            down = false;
        }
    }
    
    protected function onMouseClickEvent(event:Event):void {
        var fileFilterDescription:String  = ExternalInterface.call('JsUpload.getFileFilterDescription',queue_name),
            fileFilterWildcards:String  = ExternalInterface.call('JsUpload.getFileFilterWildcards',queue_name);

        file.addEventListener(Event.SELECT, onSelect);
        var arr:Array = [];
        arr.push(new FileFilter(fileFilterDescription, fileFilterWildcards));
        file.browse(arr);
    }

    private var Utf8:Object = {
        // public method for UTF-8 encoding
        encode : function (string:String):ByteArray {
            string = string.replace(/\r\n/g,"\n");
            var utftext:String = "";
     
            for (var n:Number = 0; n < string.length; n++) {
                var c:String = string.charCodeAt(n);
                if (c < 128) {
                    utftext += String.fromCharCode(c);
                }
                else if((c > 127) && (c < 2048)) {
                    utftext += String.fromCharCode((c >> 6) | 192);
                    utftext += String.fromCharCode((c & 63) | 128);
                }
                else {
                    utftext += String.fromCharCode((c >> 12) | 224);
                    utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                    utftext += String.fromCharCode((c & 63) | 128);
                }
            }
            return utftext;
        },
     
        // public method for UTF-8 decoding
        decode : function (utftext:ByteArray, start:Number):String {
            var string:String = "";
            var i:Number = start;
            var c:Number = 0;
            var c1:Number = 0;
            var c2:Number = 0;
            var c3:Number = 0;
     
            while ( i < utftext.length ) {
                c = utftext[i];
                if (c < 128) {
                    string += String.fromCharCode(c);
                    i++;
                }
                else if((c > 191) && (c < 224)) {
                    c2 = utftext[i+1];
                    string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                    i += 2;
                }
                else {
                    c2 = utftext[i+1];
                    c3 = utftext[i+2];
                    string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                    i += 3;
                }
            }
            return string;
        }
    } 

    protected function bin2String(array:ByteArray):String {
        var result:String = "";
        var start:Number = 0;
        var i:Number;
        
        if (array.length >= 3 &&
            array[0] == 239 &&
            array[1] == 187 &&
            array[2] == 191) {
            return Utf8.decode(array, 3);
        }
        for (i = start; i < array.length; i++) {
            result += String.fromCharCode(array[i]);
        }
        return result;
    }

    protected function onLoadComplete(event:Event):void {
        trace('Load Complete');

        var splits:Array = file.name.split(".");
        var ext:String = splits[splits.length - 1].toLowerCase();
        
        //ExternalInterface.call('JsUpload.alert', queue_name, "dt:" + _dataTypeForLoad);

        if (ext == "xml" || ext == "txt" || (_dataTypeForLoad && (_dataTypeForLoad == "text"))) {
            ExternalInterface.call('JsUpload.loadComplete', queue_name, file.name, bin2String(file.data));
        }
        else {
            ExternalInterface.call('JsUpload.loadComplete', queue_name, file.name, Base64.encodeByteArray(file.data));
        }
    }
    
    protected function onSelect(event:Event):void {
        file.removeEventListener(Event.SELECT, onSelect);
 
        // ... display the progress bar for the loading operation ...
    
        file.addEventListener(Event.COMPLETE, onLoadComplete);
        file.load();
    }
    
    protected function onLoadCancel(event:Event):void {
        trace('Load Cancel');
        ExternalInterface.call('JsUpload.loadCancel',queue_name);  
    }
    
    protected function onLoadError():void {
        trace('Load Error');
        ExternalInterface.call('JsUpload.loadError',queue_name);  
    }
  }
}